#include <iostream>
#include <assert.h>

#include "List.h"
#include "Node.h"

List::List() {
  size = 0;
  head = NULL;
}

List::List(const List& other) {
  size = other.size;
  head = NULL;

  Node* currentMe = NULL;
  Node* currentOther = other.head;

  while (currentOther != NULL) {
    Node* temp = new Node(currentOther->getItem());
    if (currentMe != NULL) {
      assert(head != NULL);
      currentMe->setNext(temp);
      currentMe = temp;
    } else {
      assert(head == NULL);
      head = currentMe = temp;
    }

    currentOther = currentOther->getNext();
  }
}

int List::getSize() const {
  return size;
}

int List::getInt(int position) const {
  assert(position >= 0 && position < size);
  Node* current = head;

  while (position > 0) {
    assert(current != NULL);
    current = current->getNext();
    position--; // count down to zero
  }

  assert(position == 0 && current != NULL);

  return current->getItem();
}

bool List::containsInt(int item) const {
  Node* current = head;

  while (current != NULL) {
    if (current->getItem() == item) {
      return true;
    }
    current = current->getNext();
  }

  return false;
}

void List::printList() const {
  Node* current = head;

  std::cout << "[";
  
  while (current != NULL) {
    Node* next = current->getNext();
    std::cout << current->getItem();

    if (next != NULL) {
      std::cout << ", ";
    }
    current = next;
  }

  std::cout << "]" << std::endl;
}

void List::addIntAtPosition(int item, int position) {
  assert(position >= 0 && position <= size);

  // go to that position in the list
  Node* previous = NULL;
  Node* current = head;

  while (position > 0) {
    assert(current != NULL);
    previous = current;
    current = current->getNext();
    position--;
  }

  assert(position == 0);

  Node* temp = new Node(item);
  temp->setNext(current);
  if (previous != NULL) {
    previous->setNext(temp);
  } else {
    // should only happen when we are inserting
    // at the head
    assert(current == head);
    head = temp;
  }

  // note that the size increased
  size++;
}

// NOTE: adding an element to the front of the list is
// much, much cheaper than putting it at the back
void List::addInt(int item) {
  // Simply call addIntAtPosition, which is more general.
  // That is, addIntAtPosition can do what addInt does, and more
  addIntAtPosition(item, size);
}

void List::removeFirstInt(int item) {
  Node* previous = NULL;
  Node* current = head;

  while (current != NULL) {
    if (current->getItem() == item) {
      if (previous != NULL) {
        previous->setNext(current->getNext());
      } else {
        assert(current == head);
        head = current->getNext();
      }
      delete current;
      current = NULL;
      // note that the size decreased
      size--;
    } else {
      previous = current;
      current = current->getNext();
    }
  }
}

List::~List() {
  Node* current = head;
  while (current != NULL) {
    Node* next = current->getNext();
    delete current;
    current = next;
  }
}
