#include "List.h"

// Important note with the code below: we quite
// intentionally say "List currentN = listBase"
// repeatedly, as opposed to saying something like:
//
// List current;
// ...
// current = listBase;
// ...
// current = listBase;
// ...
//
// What happens with `current = listBase` as opposed
// to `List current = listBase` is that the compiler
// invokes a special `=` operator on `current` with
// the parameter `listBase`.  By default, this performs
// a shallow copy, which is exactly what we are trying
// to avoid with our copy constructor!  By saying
// `List current = listBase`, we force our copy constructor
// to be used instead.

int main() {
  List listBase;

  // add elements to the list
  listBase.addInt(0);
  listBase.addInt(1);
  listBase.addInt(2);
  listBase.printList();

  // each test for addIntAtPosition
  List current1 = listBase;
  current1.addIntAtPosition(3, 0);
  current1.printList();

  List current2 = listBase;
  current2.addIntAtPosition(3, 1);
  current2.printList();

  List current3 = listBase;
  current3.addIntAtPosition(3, 2);
  current3.printList();

  List current4 = listBase;
  current4.addIntAtPosition(3, 3);
  current4.printList();

  // try out some removals
  List current5 = listBase;
  current5.removeFirstInt(1);
  current5.printList();

  return 0;
}
